@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -170,8 +170,12 @@ int main(int argc,char** argv)
        OutputJunctions(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"super"))
        OutputSuper(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
-    else if(!strcmp(option_data,"oneway"))
-       OutputOneway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
+    else if(!strcmp(option_data,"waytype-oneway"))
+       OutputWaytype(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,Highway_OneWay);
+    else if(!strcmp(option_data,"waytype-cyclebothways"))
+       OutputWaytype(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,Highway_CycleBothWays);
+    else if(!strcmp(option_data,"waytype-roundabout"))
+       OutputWaytype(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,Highway_Roundabout);
     else if(!strncmp(option_data,"highway",7) && option_data[7]=='-' && (highway=HighwayType(option_data+8))!=Highway_None)
        OutputHighway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,highway);
     else if(!strncmp(option_data,"transport",9) && option_data[9]=='-' && (transport=TransportType(option_data+10))!=Transport_None)
@@ -479,7 +483,7 @@ int main(int argc,char** argv)
          }
    }
 
- return(0);
+ exit(EXIT_SUCCESS);
 }
 
 
@@ -553,7 +557,11 @@ static void print_way(Ways *ways,index_t item)
  printf("Way %"Pindex_t"\n",item);
  if(*name)
     printf("  name=%s\n",name);
- printf("  type=%02x (%s%s%s)\n",wayp->type,HighwayName(HIGHWAY(wayp->type)),wayp->type&Highway_OneWay?",One-Way":"",wayp->type&Highway_Roundabout?",Roundabout":"");
+ printf("  type=%02x (%s%s%s%s)\n",wayp->type,
+                                   HighwayName(HIGHWAY(wayp->type)),
+                                   wayp->type&Highway_OneWay?",One-Way":"",
+                                   wayp->type&Highway_CycleBothWays?",Cycle-Both-Ways":"",
+                                   wayp->type&Highway_Roundabout?",Roundabout":"");
  printf("  allow=%02x (%s)\n",wayp->allow,AllowedNameList(wayp->allow));
  if(wayp->props)
     printf("  props=%02x (%s)\n",wayp->props,PropertiesNameList(wayp->props));
@@ -854,6 +862,9 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
  if(wayp->type & Highway_OneWay)
     printf("    <tag k='oneway' v='yes' />\n");
 
+ if(wayp->type & Highway_CycleBothWays)
+    printf("    <tag k='cyclebothways' v='yes' />\n");
+
  if(wayp->type & Highway_Roundabout)
     printf("    <tag k='roundabout' v='yes' />\n");
 
@@ -1026,6 +1037,9 @@ static void print_segment_visualiser(Segments *segments,index_t item,Ways *ways)
  if(wayp->type & Highway_OneWay)
     printf("&nbsp;&nbsp;&nbsp;&lt;tag k='oneway' v='yes' /&gt;\n");
 
+ if(wayp->type & Highway_CycleBothWays)
+    printf("&nbsp;&nbsp;&nbsp;&lt;tag k='cyclebothways' v='yes' /&gt;\n");
+
  if(wayp->type & Highway_Roundabout)
     printf("&nbsp;&nbsp;&nbsp;&lt;tag k='roundabout' v='yes' /&gt;\n");
 
@@ -1227,7 +1241,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "  <data-type> can be selected from:\n"
             "      junctions   = segment count at each junction.\n"
             "      super       = super-node and super-segments.\n"
-            "      oneway      = oneway segments.\n"
+            "      waytype-*   = segments of oneway, cyclebothways or roundabout type.\n"
             "      highway-*   = segments of the specified highway type.\n"
             "      transport-* = segments allowing the specified transport type.\n"
             "      barrier-*   = nodes disallowing the specified transport type.\n"
