@@ -142,6 +142,8 @@ void *MapFile(const char *filename)
     exit(EXIT_FAILURE);
    }
 
+ log_mmap(size);
+
  /* Store the information about the mapped file */
 
  mappedfiles=(struct mmapinfo*)realloc((void*)mappedfiles,(nmappedfiles+1)*sizeof(struct mmapinfo));
@@ -204,6 +206,8 @@ void *MapFileWriteable(const char *filename)
     exit(EXIT_FAILURE);
    }
 
+ log_mmap(size);
+
  /* Store the information about the mapped file */
 
  mappedfiles=(struct mmapinfo*)realloc((void*)mappedfiles,(nmappedfiles+1)*sizeof(struct mmapinfo));
@@ -249,6 +253,8 @@ void *UnmapFile(const void *address)
 
  munmap(mappedfiles[i].address,mappedfiles[i].length);
 
+ log_munmap(mappedfiles[i].length);
+
  /* Shuffle the list of files */
 
  nmappedfiles--;
@@ -618,6 +624,28 @@ off_t SizeFile(const char *filename)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Get the size of a file from a file descriptor.
+
+  off_t SizeFileFD Returns the file size if OK or exits in case of an error.
+
+  int fd The file descriptor to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+off_t SizeFileFD(int fd)
+{
+ struct stat buf;
+
+ if(fstat(fd,&buf))
+   {
+    fprintf(stderr,"Cannot stat file descriptor '%d' [%s].\n",fd,strerror(errno));
+    exit(EXIT_FAILURE);
+   }
+
+ return(buf.st_size);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Check if a file exists.
 
