@@ -108,6 +108,8 @@ int main(int argc,char** argv)
        option_loggable=1;
     else if(!strcmp(argv[arg],"--logtime"))
        option_logtime=1;
+    else if(!strcmp(argv[arg],"--logmemory"))
+       option_logmemory=1;
     else if(!strcmp(argv[arg],"--errorlog"))
        errorlog="error.log";
     else if(!strncmp(argv[arg],"--errorlog=",11))
@@ -313,7 +315,7 @@ if(!option_process_only)
     FreeWayList(OSMWays,1);
     FreeRelationList(OSMRelations,1);
 
-    return(0);
+    exit(EXIT_SUCCESS);
    }
 
 
@@ -335,11 +337,11 @@ if(!option_process_only)
  printf("\nProcess OSM Data\n================\n\n");
  fflush(stdout);
 
- /* Remove non-highway nodes by looking through the ways */
+ /* Remove non-highway nodes by looking through the ways (must be before splitting the ways) */
 
  RemoveNonHighwayNodes(OSMNodes,OSMWays,option_keep||option_changes);
 
- /* Separate the segments and way names and sort them. */
+ /* Separate the segments and way names and sort them (must be before processing the segments) */
 
  OSMSegments=SplitWays(OSMWays,OSMNodes,option_keep||option_changes);
 
@@ -347,7 +349,7 @@ if(!option_process_only)
 
  SortSegmentList(OSMSegments);
 
- /* Process the segments and index them */
+ /* Process the segments and index them (must be before processing relations) */
 
  ProcessSegments(OSMSegments,OSMNodes,OSMWays);
 
@@ -359,42 +361,78 @@ if(!option_process_only)
 
  ProcessTurnRelations(OSMRelations,OSMNodes,OSMSegments,OSMWays,option_keep||option_changes);
 
- /* Compact the ways (must be after processing turn relations) */
+ /* Compact the ways */
 
  CompactWayList(OSMWays,OSMSegments);
 
- /* Index the segments */
+ /* Sort the nodes and segments geographically */
+
+ SortNodeListGeographically(OSMNodes);
+
+ SortSegmentListGeographically(OSMSegments,OSMNodes);
+
+ /* Re-index the segments */
 
  IndexSegments(OSMSegments,OSMNodes,OSMWays);
 
- /* Prune unwanted nodes/segments. */
+ /* Sort the turn relations geographically */
+
+ SortTurnRelationListGeographically(OSMRelations,OSMNodes,OSMSegments);
+
+ /* Prune unwanted nodes/segments */
 
  if(option_prune_straight || option_prune_isolated || option_prune_short)
    {
     printf("\nPrune Unneeded Data\n===================\n\n");
     fflush(stdout);
 
-    StartPruning(OSMNodes,OSMSegments,OSMWays);
-
     if(option_prune_straight)
+      {
+       StartPruning(OSMNodes,OSMSegments,OSMWays);
+
        PruneStraightHighwayNodes(OSMNodes,OSMSegments,OSMWays,option_prune_straight);
 
+       FinishPruning(OSMNodes,OSMSegments,OSMWays);
+
+       RemovePrunedNodes(OSMNodes,OSMSegments);
+       RemovePrunedSegments(OSMSegments,OSMWays);
+       CompactWayList(OSMWays,OSMSegments);
+       RemovePrunedTurnRelations(OSMRelations,OSMNodes);
+
+       IndexSegments(OSMSegments,OSMNodes,OSMWays);
+      }
+
     if(option_prune_isolated)
+      {
+       StartPruning(OSMNodes,OSMSegments,OSMWays);
+
        PruneIsolatedRegions(OSMNodes,OSMSegments,OSMWays,option_prune_isolated);
 
+       FinishPruning(OSMNodes,OSMSegments,OSMWays);
+
+       RemovePrunedNodes(OSMNodes,OSMSegments);
+       RemovePrunedSegments(OSMSegments,OSMWays);
+       CompactWayList(OSMWays,OSMSegments);
+       RemovePrunedTurnRelations(OSMRelations,OSMNodes);
+
+       IndexSegments(OSMSegments,OSMNodes,OSMWays);
+      }
+
     if(option_prune_short)
+      {
+       StartPruning(OSMNodes,OSMSegments,OSMWays);
+
        PruneShortSegments(OSMNodes,OSMSegments,OSMWays,option_prune_short);
 
-    FinishPruning(OSMNodes,OSMSegments,OSMWays);
+       FinishPruning(OSMNodes,OSMSegments,OSMWays);
 
-    /* Remove the pruned nodes, segments, ways and relations and update the indexes */
+       RemovePrunedNodes(OSMNodes,OSMSegments);
+       RemovePrunedSegments(OSMSegments,OSMWays);
+       CompactWayList(OSMWays,OSMSegments);
+       RemovePrunedTurnRelations(OSMRelations,OSMNodes);
 
-    RemovePrunedNodes(OSMNodes,OSMSegments);
-    RemovePrunedSegments(OSMSegments,OSMWays);
-    CompactWayList(OSMWays,OSMSegments);
-    RemovePrunedTurnRelations(OSMRelations,OSMNodes);
-
-    IndexSegments(OSMSegments,OSMNodes,OSMWays);
+       IndexSegments(OSMSegments,OSMNodes,OSMWays);
+      }
    }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
@@ -422,6 +460,10 @@ if(!option_process_only)
       {
        SegmentsX *SuperSegments2;
 
+       /* Index the super-segments */
+
+       IndexSegments(SuperSegments,OSMNodes,OSMWays);
+
        /* Select the super-nodes */
 
        ChooseSuperNodes(OSMNodes,SuperSegments,OSMWays);
@@ -441,10 +483,6 @@ if(!option_process_only)
 
     DeduplicateSuperSegments(SuperSegments,OSMWays);
 
-    /* Index the segments */
-
-    IndexSegments(SuperSegments,OSMNodes,OSMWays);
-
     /* Check for end condition */
 
     if(SuperSegments->number==nsuper)
@@ -512,6 +550,10 @@ if(!option_process_only)
 
  SaveRelationList(OSMRelations,FileName(dirname,prefix,"relations.mem"));
 
+ /* Free the memory (delete the temporary files) */
+
+ FreeSegmentList(OSMSegments);
+
  /* Close the error log file and process the data */
 
  if(errorlog)
@@ -543,11 +585,12 @@ if(!option_process_only)
  FreeWayList(OSMWays,0);
  FreeRelationList(OSMRelations,0);
 
- FreeSegmentList(OSMSegments);
+ printf("\n");
+ fflush(stdout);
 
  printf_program_end();
 
- return(0);
+ exit(EXIT_SUCCESS);
 }
 
 
@@ -573,7 +616,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
 #endif
          "                      [--tmpdir=<dirname>]\n"
          "                      [--tagging=<filename>]\n"
-         "                      [--loggable] [--logtime]\n"
+         "                      [--loggable] [--logtime] [--logmemory]\n"
          "                      [--errorlog[=<name>]]\n"
          "                      [--parse-only | --process-only]\n"
          "                      [--append] [--keep] [--changes]\n"
@@ -634,6 +677,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "\n"
             "--loggable                Print progress messages suitable for logging to file.\n"
             "--logtime                 Print the elapsed time for each processing step.\n"
+            "--logmemory               Print the max allocated/mapped memory for each step.\n"
             "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
             "                          (the '--dir' and '--prefix' options are applied).\n"
             "\n"
