@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2013 Andrew M. Bishop
+ This file Copyright 2010-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -57,6 +57,7 @@ static int sort_turn_by_id(TurnRelX *a,TurnRelX *b);
 static int deduplicate_turn_by_id(TurnRelX *relationx,index_t index);
 
 static int geographically_index(TurnRelX *relationx,index_t index);
+static int geographically_index_convert_segments(TurnRelX *relationx,index_t index);
 static int sort_by_via(TurnRelX *a,TurnRelX *b);
 
 
@@ -167,10 +168,16 @@ void FreeRelationList(RelationsX *relationsx,int keep)
  free(relationsx->rrfilename_tmp);
 
  if(relationsx->rridata)
+   {
+    log_free(relationsx->rridata);
     free(relationsx->rridata);
+   }
 
  if(relationsx->rrodata)
+   {
+    log_free(relationsx->rrodata);
     free(relationsx->rrodata);
+   }
 
 
  /* Turn Restriction relations */
@@ -184,7 +191,10 @@ void FreeRelationList(RelationsX *relationsx,int keep)
  free(relationsx->trfilename_tmp);
 
  if(relationsx->tridata)
+   {
+    log_free(relationsx->tridata);
     free(relationsx->tridata);
+   }
 
 
  free(relationsx);
@@ -1116,12 +1126,18 @@ void ProcessTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segme
 
  /* Free the now-unneeded indexes */
 
+ log_free(nodesx->idata);
  free(nodesx->idata);
  nodesx->idata=NULL;
 
+ log_free(waysx->idata);
  free(waysx->idata);
  waysx->idata=NULL;
 
+ log_free(segmentsx->firstnode);
+ free(segmentsx->firstnode);
+ segmentsx->firstnode=NULL;
+
  /* Unmap from memory / close the files */
 
 #if !SLIM
@@ -1219,6 +1235,7 @@ void RemovePrunedTurnRelations(RelationsX *relationsx,NodesX *nodesx)
 
 void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx)
 {
+ static int which=1;
  int trfd;
 
  if(segmentsx->number==0)
@@ -1251,9 +1268,16 @@ void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,Se
  sortnodesx=nodesx;
  sortsegmentsx=segmentsx;
 
- filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRelX),(int (*)(void*,index_t))geographically_index,
-                                                       (int (*)(const void*,const void*))sort_by_via,
-                                                       NULL);
+ if(which==1)
+    filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRelX),(int (*)(void*,index_t))geographically_index,
+                                                          (int (*)(const void*,const void*))sort_by_via,
+                                                          NULL);
+ else
+    filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRelX),(int (*)(void*,index_t))geographically_index_convert_segments,
+                                                          (int (*)(const void*,const void*))sort_by_via,
+                                                          NULL);
+
+ which++;
 
  /* Close the files */
 
@@ -1272,6 +1296,7 @@ void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,Se
 
  if(nodesx->gdata)
    {
+    log_free(nodesx->gdata);
     free(nodesx->gdata);
     nodesx->gdata=NULL;
    }
@@ -1293,6 +1318,26 @@ void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,Se
   ++++++++++++++++++++++++++++++++++++++*/
 
 static int geographically_index(TurnRelX *relationx,index_t index)
+{
+ relationx->from=sortnodesx->gdata[relationx->from];
+ relationx->via =sortnodesx->gdata[relationx->via];
+ relationx->to  =sortnodesx->gdata[relationx->to];
+
+ return(1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update the turn relation indexes and replace them with segments.
+
+  int geographically_index_convert_segments Return 1 if the value is to be kept, otherwise 0.
+
+  TurnRelX *relationx The extended turn relation.
+
+  index_t index The number of unsorted turn relations that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int geographically_index_convert_segments(TurnRelX *relationx,index_t index)
 {
  SegmentX *segmentx;
  index_t from_node,via_node,to_node;
