@@ -28,7 +28,6 @@
 #include "segments.h"
 #include "ways.h"
 
-#include "cache.h"
 #include "fakes.h"
 #include "files.h"
 #include "profiles.h"
@@ -69,6 +68,7 @@ Segments *LoadSegmentList(const char *filename)
  SlimFetch(segments->fd,&segments->file,sizeof(SegmentsFile),0);
 
  segments->cache=NewSegmentCache();
+ log_malloc(segments->cache,sizeof(*segments->cache));
 
 #endif
 
@@ -92,6 +92,7 @@ void DestroySegmentList(Segments *segments)
 
  segments->fd=SlimUnmapFile(segments->fd);
 
+ log_free(segments->cache);
  DeleteSegmentCache(segments->cache);
 
 #endif
@@ -159,7 +160,7 @@ index_t FindClosestSegmentHeading(Nodes *nodes,Segments *segments,Ways *ways,ind
        if(profile->allow!=Transports_Bicycle)
           goto endloop;
 
-       if(!(wayp->props&Properties_CycleBothWays))
+       if(!(wayp->type&Highway_CycleBothWays))
           goto endloop;
       }
 
@@ -218,7 +219,7 @@ distance_t Distance(double lat1,double lon1,double lat2,double lon2)
 
  a1 = sin (dlat / 2);
  a2 = sin (dlon / 2);
- a = (a1 * a1) + cos (lat1) * cos (lat2) * a2 * a2;
+ a = a1 * a1 + cos (lat1) * cos (lat2) * a2 * a2;
  sa = sqrt (a);
  if (sa <= 1.0)
    {c = 2 * asin (sa);}
@@ -230,6 +231,68 @@ distance_t Distance(double lat1,double lon1,double lat2,double lon2)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Calculate the change in latitude (same longitude) between two locations a known distance apart.
+
+  double DeltaLat Returns the difference in latitude between the locations.
+
+  double lon The longitude of the locations.
+
+  distance_t distance The distance between the locations.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+double DeltaLat(double lon,distance_t distance)
+{
+ double dlat;
+
+ double c,d;
+
+ if(distance==0)
+   return 0;
+
+ d = distance_to_km(distance);
+
+ c = d / 6378.137;
+
+ dlat = c;
+
+ return dlat;
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Calculate the change in longitude (same latitude) between two locations a known distance apart.
+
+  double DeltaLon Returns the difference in longitude between the locations.
+
+  double lat The latitude of the locations.
+
+  distance_t distance The distance between the locations.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+double DeltaLon(double lat,distance_t distance)
+{
+ double dlon;
+
+ double a2,sa,c,d;
+
+ if(distance==0)
+   return 0;
+
+ d = distance_to_km(distance);
+
+ c = d / 6378.137;
+
+ sa = sin(c/2);
+
+ a2 = sa / cos(lat);
+
+ dlon = 2*asin(a2);
+
+ return dlon;
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Calculate the duration of travel on a segment.
 
