@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -85,6 +85,7 @@ SegmentsX *NewSegmentList(void)
 
 #if SLIM
  segmentsx->cache=NewSegmentXCache();
+ log_malloc(segmentsx->cache,sizeof(*segmentsx->cache));
 #endif
 
  return(segmentsx);
@@ -103,13 +104,26 @@ void FreeSegmentList(SegmentsX *segmentsx)
 
  free(segmentsx->filename_tmp);
 
+ if(segmentsx->usedway)
+   {
+    log_free(segmentsx->usedway);
+    free(segmentsx->usedway);
+   }
+
  if(segmentsx->firstnode)
+   {
+    log_free(segmentsx->firstnode);
     free(segmentsx->firstnode);
+   }
 
  if(segmentsx->next1)
+   {
+    log_free(segmentsx->next1);
     free(segmentsx->next1);
+   }
 
 #if SLIM
+ log_free(segmentsx->cache);
  DeleteSegmentXCache(segmentsx->cache);
 #endif
 
@@ -384,18 +398,16 @@ void ProcessSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
 #if !SLIM
  nodesx->data=MapFile(nodesx->filename_tmp);
- waysx->data=MapFile(waysx->filename_tmp);
 #else
  nodesx->fd=SlimMapFile(nodesx->filename_tmp);
- waysx->fd=SlimMapFile(waysx->filename_tmp);
 
  InvalidateNodeXCache(nodesx->cache);
- InvalidateWayXCache(waysx->cache);
 #endif
 
  /* Allocate the way usage bitmask */
 
  segmentsx->usedway=AllocBitMask(waysx->number);
+ log_malloc(segmentsx->usedway,LengthBitMask(waysx->number)*sizeof(BitMask));
 
  logassert(segmentsx->usedway,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
@@ -411,36 +423,39 @@ void ProcessSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
  while(!ReadFileBuffered(segmentsx->fd,&segmentx,sizeof(SegmentX)))
    {
-    NodeX *nodex1=LookupNodeX(nodesx,segmentx.node1,1);
-    NodeX *nodex2=LookupNodeX(nodesx,segmentx.node2,2);
-
     if(prevnode1==segmentx.node1 && prevnode2==segmentx.node2)
       {
+       node_t id1=nodesx->idata[segmentx.node1];
+       node_t id2=nodesx->idata[segmentx.node2];
+
        if(prevway==segmentx.way)
          {
-          WayX *wayx=LookupWayX(waysx,segmentx.way,1);
+          way_t id=waysx->idata[segmentx.way];
 
-          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" in way %"Pway_t" is duplicated.\n",logerror_node(nodex1->id),logerror_node(nodex2->id),logerror_way(wayx->id));
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" in way %"Pway_t" is duplicated.\n",logerror_node(id1),logerror_node(id2),logerror_way(id));
          }
        else
          {
           if(!(prevdist&SEGMENT_AREA) && !(segmentx.distance&SEGMENT_AREA))
-             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",logerror_node(nodex1->id),logerror_node(nodex2->id));
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",logerror_node(id1),logerror_node(id2));
 
           if(!(prevdist&SEGMENT_AREA) && (segmentx.distance&SEGMENT_AREA))
-             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the area).\n",logerror_node(nodex1->id),logerror_node(nodex2->id));
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the area).\n",logerror_node(id1),logerror_node(id2));
 
           if((prevdist&SEGMENT_AREA) && !(segmentx.distance&SEGMENT_AREA))
-             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the non-area).\n",logerror_node(nodex1->id),logerror_node(nodex2->id));
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the non-area).\n",logerror_node(id1),logerror_node(id2));
 
           if((prevdist&SEGMENT_AREA) && (segmentx.distance&SEGMENT_AREA))
-             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (both are areas).\n",logerror_node(nodex1->id),logerror_node(nodex2->id));
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (both are areas).\n",logerror_node(id1),logerror_node(id2));
          }
 
        duplicate++;
       }
     else
       {
+       NodeX *nodex1=LookupNodeX(nodesx,segmentx.node1,1);
+       NodeX *nodex2=LookupNodeX(nodesx,segmentx.node2,2);
+
        prevnode1=segmentx.node1;
        prevnode2=segmentx.node2;
        prevway=segmentx.way;
@@ -479,10 +494,8 @@ void ProcessSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
 #if !SLIM
  nodesx->data=UnmapFile(nodesx->data);
- waysx->data=UnmapFile(waysx->data);
 #else
  nodesx->fd=SlimUnmapFile(nodesx->fd);
- waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
  /* Print the final message */
@@ -518,10 +531,8 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
  /* Allocate the array of indexes */
 
- if(segmentsx->firstnode)
-    free(segmentsx->firstnode);
-
  segmentsx->firstnode=(index_t*)malloc(nodesx->number*sizeof(index_t));
+ log_malloc(segmentsx->firstnode,nodesx->number*sizeof(index_t));
 
  logassert(segmentsx->firstnode,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
@@ -596,12 +607,14 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
  if(nodesx->pdata)
    {
+    log_free(nodesx->pdata);
     free(nodesx->pdata);
     nodesx->pdata=NULL;
    }
 
  if(waysx->cdata)
    {
+    log_free(waysx->cdata);
     free(waysx->cdata);
     waysx->cdata=NULL;
    }
@@ -635,6 +648,7 @@ void RemovePrunedSegments(SegmentsX *segmentsx,WaysX *waysx)
  /* Allocate the way usage bitmask */
 
  segmentsx->usedway=AllocBitMask(waysx->number);
+ log_malloc(segmentsx->usedway,LengthBitMask(waysx->number)*sizeof(BitMask));
 
  logassert(segmentsx->usedway,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
@@ -997,7 +1011,7 @@ static distance_t DistanceX(NodeX *nodex1,NodeX *nodex2)
 
  a1 = sin (dlat / 2);
  a2 = sin (dlon / 2);
- a = (a1 * a1) + cos (lat1) * cos (lat2) * a2 * a2;
+ a = a1 * a1 + cos (lat1) * cos (lat2) * a2 * a2;
  sa = sqrt (a);
  if (sa <= 1.0)
    {c = 2 * asin (sa);}
