@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2009-2013 Andrew M. Bishop
+ This file Copyright 2009-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -121,7 +121,7 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
  int *fds=NULL,*heap=NULL;
  int nfiles=0,ndata=0;
  index_t count_out=0,count_in=0,total=0;
- size_t nitems=option_filesort_ramsize/(option_filesort_threads*(itemsize+sizeof(void*)));
+ size_t nitems;
  void *data,**datap;
  thread_data *threads;
  size_t item;
@@ -132,6 +132,16 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
  /* Allocate the RAM buffer and other bits */
 
+ nitems=SizeFileFD(fd_in)/itemsize;
+
+ if(nitems==0)
+    return(0);
+
+ if((nitems*(itemsize+sizeof(void*)))<option_filesort_ramsize)
+    nitems=1+nitems/option_filesort_threads;
+ else
+    nitems=option_filesort_ramsize/(option_filesort_threads*(itemsize+sizeof(void*)));
+
  threads=(thread_data*)malloc(option_filesort_threads*sizeof(thread_data));
 
  for(i=0;i<option_filesort_threads;i++)
@@ -141,6 +151,9 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
     threads[i].data=malloc(nitems*itemsize);
     threads[i].datap=malloc(nitems*sizeof(void*));
 
+    log_malloc(threads[i].data,nitems*itemsize);
+    log_malloc(threads[i].datap,nitems*sizeof(void*));
+
     threads[i].filename=(char*)malloc(strlen(option_tmpdirname)+24);
 
     threads[i].itemsize=itemsize;
@@ -435,6 +448,9 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
  for(i=0;i<option_filesort_threads;i++)
    {
+    log_free(threads[i].data);
+    log_free(threads[i].datap);
+
     free(threads[i].data);
     free(threads[i].datap);
 
@@ -484,7 +500,7 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
  int *fds=NULL,*heap=NULL;
  int nfiles=0,ndata=0;
  index_t count_out=0,count_in=0,total=0;
- size_t datasize=option_filesort_ramsize/option_filesort_threads;
+ size_t datasize;
  FILESORT_VARINT nextitemsize,largestitemsize=0;
  void *data,**datap;
  thread_data *threads;
@@ -496,6 +512,16 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
  /* Allocate the RAM buffer and other bits */
 
+ datasize=SizeFileFD(fd_in);
+
+ if(datasize==0)
+    return(0);
+
+ if((datasize*2)<option_filesort_ramsize) /* estimate that data and pointer are same size */
+    datasize=(datasize*2)/option_filesort_threads;
+ else
+    datasize=option_filesort_ramsize/option_filesort_threads;
+
  threads=(thread_data*)malloc(option_filesort_threads*sizeof(thread_data));
 
  for(i=0;i<option_filesort_threads;i++)
@@ -505,6 +531,8 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
     threads[i].data=malloc(datasize);
     threads[i].datap=NULL;
 
+    log_malloc(threads[i].data,datasize);
+
     threads[i].filename=(char*)malloc(strlen(option_tmpdirname)+24);
 
     threads[i].compare=compare_function;
@@ -838,6 +866,8 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
  for(i=0;i<option_filesort_threads;i++)
    {
+    log_free(threads[i].data);
+
     free(threads[i].data);
 
     free(threads[i].filename);
