@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -69,6 +69,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  if(!nodesx->super)
    {
     nodesx->super=AllocBitMask(nodesx->number);
+    log_malloc(nodesx->super,LengthBitMask(nodesx->number)*sizeof(BitMask));
 
     logassert(nodesx->super,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
@@ -327,6 +328,15 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
+ /* Free the no-longer required memory */
+
+ if(segmentsx->firstnode)
+   {
+    log_free(segmentsx->firstnode);
+    free(segmentsx->firstnode);
+    segmentsx->firstnode=NULL;
+   }
+
  /* Print the final message */
 
  printf_last("Created Super-Segments: Super-Nodes=%"Pindex_t" Super-Segments=%"Pindex_t,sn,ss);
@@ -350,6 +360,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
  index_t i,j,lastj;
  index_t merged=0,added=0;
  SegmentsX *mergedsegmentsx;
+ SegmentX supersegmentx;
 
  mergedsegmentsx=NewSegmentList();
 
@@ -384,8 +395,6 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 
     while(j<supersegmentsx->number)
       {
-       SegmentX supersegmentx;
-
        if(j!=lastj)
          {
           ReadFileBuffered(supersegmentsx->fd,&supersegmentx,sizeof(SegmentX));
@@ -429,15 +438,24 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
        printf_middle("Merging Segments: Segments=%"Pindex_t" Super=%"Pindex_t" Merged=%"Pindex_t" Added=%"Pindex_t,i+1,j,merged,added);
    }
 
- while(j<supersegmentsx->number)
+ if(j<supersegmentsx->number)
    {
-    SegmentX supersegmentx;
+    if(j==lastj)
+      {
+       AppendSegmentList(mergedsegmentsx,supersegmentx.way,supersegmentx.node1,supersegmentx.node2,supersegmentx.distance|SEGMENT_SUPER);
 
-    ReadFileBuffered(supersegmentsx->fd,&supersegmentx,sizeof(SegmentX));
+       j++;
+      }
 
-    AppendSegmentList(mergedsegmentsx,supersegmentx.way,supersegmentx.node1,supersegmentx.node2,supersegmentx.distance|SEGMENT_SUPER);
-    added++;
-    j++;
+    while(j<supersegmentsx->number)
+      {
+       ReadFileBuffered(supersegmentsx->fd,&supersegmentx,sizeof(SegmentX));
+
+       AppendSegmentList(mergedsegmentsx,supersegmentx.way,supersegmentx.node1,supersegmentx.node2,supersegmentx.distance|SEGMENT_SUPER);
+
+       added++;
+       j++;
+      }
    }
 
  FinishSegmentList(mergedsegmentsx);
