@@ -65,6 +65,7 @@ static int limit_type=0;
 static Highway highways=Highway_None;
 static Transports transports=Transports_None;
 static Properties properties=Properties_None;
+static highway_t waytype=0;
 
 /* Local functions */
 
@@ -72,7 +73,7 @@ static void find_all_nodes(Nodes *nodes,callback_t callback);
 
 static void output_junctions(index_t node,double latitude,double longitude);
 static void output_super(index_t node,double latitude,double longitude);
-static void output_oneway(index_t node,double latitude,double longitude);
+static void output_waytype(index_t node,double latitude,double longitude);
 static void output_highway(index_t node,double latitude,double longitude);
 static void output_transport(index_t node,double latitude,double longitude);
 static void output_barrier(index_t node,double latitude,double longitude);
@@ -82,7 +83,7 @@ static void output_property(index_t node,double latitude,double longitude);
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for junctions.
+  Output the data for junctions (--data=junctions).
 
   Nodes *nodes The set of nodes to use.
 
@@ -122,7 +123,7 @@ void OutputJunctions(Nodes *nodes,Segments *segments,Ways *ways,Relations *relat
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node as a junction (called as a callback).
+  Process a single node and output all those that are junctions (called as a callback).
 
   index_t node The node to output.
 
@@ -161,7 +162,7 @@ static void output_junctions(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for super-nodes and super-segments.
+  Output the data for super-nodes and super-segments (--data=super).
 
   Nodes *nodes The set of nodes to use.
 
@@ -201,7 +202,7 @@ void OutputSuper(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node as a super-node (called as a callback).
+  Process a single node and output all that are super-nodes and all connected super-segments (called as a callback).
 
   index_t node The node to output.
 
@@ -242,7 +243,7 @@ static void output_super(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for one-way segments.
+  Output the data for segments of special highway types (--data=waytype-oneway etc).
 
   Nodes *nodes The set of nodes to use.
 
@@ -259,9 +260,11 @@ static void output_super(index_t node,double latitude,double longitude)
   double lonmin The minimum longitude.
 
   double lonmax The maximum longitude.
+
+  highway_t mask A bit mask that must match the highway type.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void OutputOneway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax)
+void OutputWaytype(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax,highway_t mask)
 {
  /* Use local variables so that the callback doesn't need to pass them backwards and forwards */
 
@@ -277,12 +280,14 @@ void OutputOneway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relation
 
  /* Iterate through the nodes and process them */
 
- find_all_nodes(nodes,(callback_t)output_oneway);
+ waytype=mask;
+
+ find_all_nodes(nodes,(callback_t)output_waytype);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node and all connected one-way segments (called as a callback).
+  Process a single node and output all connected segments of a particular special highway type (called as a callback).
 
   index_t node The node to output.
 
@@ -291,7 +296,7 @@ void OutputOneway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relation
   double longitude The longitude of the node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void output_oneway(index_t node,double latitude,double longitude)
+static void output_waytype(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
  Segment *segmentp;
@@ -302,18 +307,22 @@ static void output_oneway(index_t node,double latitude,double longitude)
    {
     if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segmentp,node);
+       Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-       if(node>othernode)
+       if(wayp->type&waytype)
          {
+          index_t othernode=OtherNode(segmentp,node);
           double lat,lon;
 
           GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
-          if(IsOnewayFrom(segmentp,node))
-             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
-          else if(IsOnewayFrom(segmentp,othernode))
-             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(lat),radians_to_degrees(lon),radians_to_degrees(latitude),radians_to_degrees(longitude));
+          if(node>othernode || (lat<LatMin || lat>LatMax || lon<LonMin || lon>LonMax))
+            {
+             if(IsOnewayFrom(segmentp,node))
+                printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+             else if(IsOnewayFrom(segmentp,othernode))
+                printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(lat),radians_to_degrees(lon),radians_to_degrees(latitude),radians_to_degrees(longitude));
+            }
          }
       }
 
@@ -324,7 +333,7 @@ static void output_oneway(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for segments of a particular highway type.
+  Output the data for segments of a particular highway type (--data=highway-primary etc).
 
   Nodes *nodes The set of nodes to use.
 
@@ -368,7 +377,7 @@ void OutputHighway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relatio
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node and all connected one-way segments (called as a callback).
+  Process a single node and output all connected segments that are of a particular highway type (called as a callback).
 
   index_t node The node to output.
 
@@ -388,20 +397,17 @@ static void output_highway(index_t node,double latitude,double longitude)
    {
     if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segmentp,node);
+       Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-       if(node>othernode)
+       if(HIGHWAY(wayp->type)==highways)
          {
-          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
+          index_t othernode=OtherNode(segmentp,node);
+          double lat,lon;
 
-          if(HIGHWAY(wayp->type)==highways)
-            {
-             double lat,lon;
-
-             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
+          GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
+          if(node>othernode || (lat<LatMin || lat>LatMax || lon<LonMin || lon>LonMax))
              printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
-            }
          }
       }
 
@@ -412,7 +418,7 @@ static void output_highway(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for segments allowed for a particular type of traffic.
+  Output the data for segments allowed for a particular type of traffic (--data=transport-motorcar etc).
 
   Nodes *nodes The set of nodes to use.
 
@@ -456,7 +462,7 @@ void OutputTransport(Nodes *nodes,Segments *segments,Ways *ways,Relations *relat
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node and all connected one-way segments (called as a callback).
+  Process a single node and output all connected segments for a particular traffic type (called as a callback).
 
   index_t node The node to output.
 
@@ -476,20 +482,17 @@ static void output_transport(index_t node,double latitude,double longitude)
    {
     if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segmentp,node);
+       Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-       if(node>othernode)
+       if(wayp->allow&transports)
          {
-          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
+          index_t othernode=OtherNode(segmentp,node);
+          double lat,lon;
 
-          if(wayp->allow&transports)
-            {
-             double lat,lon;
-
-             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
+          GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
+          if(node>othernode || (lat<LatMin || lat>LatMax || lon<LonMin || lon>LonMax))
              printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
-            }
          }
       }
 
@@ -500,7 +503,7 @@ static void output_transport(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for nodes disallowed for a particular type of traffic.
+  Output the data for nodes disallowed for a particular type of traffic (--data=barrier).
 
   Nodes *nodes The set of nodes to use.
 
@@ -544,7 +547,7 @@ void OutputBarrier(Nodes *nodes,Segments *segments,Ways *ways,Relations *relatio
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node (called as a callback).
+  Process a single node and output those that are barriers (called as a callback).
 
   index_t node The node to output.
 
@@ -563,7 +566,7 @@ static void output_barrier(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for turn restrictions.
+  Output the data for turn restrictions (--data=turns).
 
   Nodes *nodes The set of nodes to use.
 
@@ -603,7 +606,7 @@ void OutputTurnRestrictions(Nodes *nodes,Segments *segments,Ways *ways,Relations
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node as the 'via' node for a turn restriction (called as a callback).
+  Process a single node and output those that are 'via' nodes for a turn restriction and the associated segments (called as a callback).
 
   index_t node The node to output.
 
@@ -653,7 +656,7 @@ static void output_turnrestriction(index_t node,double latitude,double longitude
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for speed limits.
+  Output the data for speed limits (--data=speed).
 
   Nodes *nodes The set of nodes to use.
 
@@ -695,7 +698,7 @@ void OutputSpeedLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for weight limits.
+  Output the data for weight limits (--data=weight).
 
   Nodes *nodes The set of nodes to use.
 
@@ -737,7 +740,7 @@ void OutputWeightLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *re
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for height limits.
+  Output the data for height limits (--data=height).
 
   Nodes *nodes The set of nodes to use.
 
@@ -779,7 +782,7 @@ void OutputHeightLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *re
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for width limits.
+  Output the data for width limits (--data=width).
 
   Nodes *nodes The set of nodes to use.
 
@@ -821,7 +824,7 @@ void OutputWidthLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for length limits.
+  Output the data for length limits (--data=length).
 
   Nodes *nodes The set of nodes to use.
 
@@ -863,7 +866,7 @@ void OutputLengthLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *re
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node as a speed, weight, height, length, width limit (called as a callback).
+  Process a single node and output those and connected segments that have a speed, weight, height, width or length limit change (called as a callback).
 
   index_t node The node to output.
 
@@ -965,7 +968,7 @@ static void output_limits(index_t node,double latitude,double longitude)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for segments that have a particular property.
+  Output the data for segments that have a particular property (--data=property-paved etc).
 
   Nodes *nodes The set of nodes to use.
 
@@ -1009,7 +1012,7 @@ void OutputProperty(Nodes *nodes,Segments *segments,Ways *ways,Relations *relati
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Process a single node and all connected one-way segments (called as a callback).
+  Process a single node and output all connected segments with a particular property (called as a callback).
 
   index_t node The node to output.
 
@@ -1029,20 +1032,17 @@ static void output_property(index_t node,double latitude,double longitude)
    {
     if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segmentp,node);
+       Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-       if(node>othernode)
+       if(wayp->props&properties)
          {
-          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
+          index_t othernode=OtherNode(segmentp,node);
+          double lat,lon;
 
-          if(wayp->props&properties)
-            {
-             double lat,lon;
-
-             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
+          GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
+          if(node>othernode || (lat<LatMin || lat>LatMax || lon<LonMin || lon>LonMax))
              printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
-            }
          }
       }
 
@@ -1103,7 +1103,7 @@ static void find_all_nodes(Nodes *nodes,callback_t callback)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for error logs within the region.
+  Output the data for error logs within the region (--data=errorlogs).
 
   ErrorLogs *errorlogs The set of error logs to use.
 
