@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -24,7 +24,6 @@
 
 #include "ways.h"
 
-#include "cache.h"
 #include "files.h"
 
 
@@ -66,6 +65,7 @@ Ways *LoadWayList(const char *filename)
  ways->namesoffset=sizeof(WaysFile)+ways->file.number*sizeof(Way);
 
  ways->cache=NewWayCache();
+ log_malloc(ways->cache,sizeof(*ways->cache));
 
 #endif
 
@@ -89,6 +89,7 @@ void DestroyWayList(Ways *ways)
 
  ways->fd=SlimUnmapFile(ways->fd);
 
+ log_free(ways->cache);
  DeleteWayCache(ways->cache);
 
 #endif
